/* test pma with persistent linked list, illustrate root pointer
   Copyright (C) 2022  Terence Kelly
   Contact:  tpkelly @ { acm.org, cs.princeton.edu, eecs.umich.edu }

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Affero General Public License for more details.

   You should have received a copy of the GNU Affero General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef NO_PMA     // By default, compile with pma library
#include "pma.h"   // unless overridden via -D
#else
#define pma_malloc       malloc     // These #defines
#define pma_free         free       // neutralize pma_*
#define pma_init(x,y)    0          // functions to enable
#define pma_errno        0          // Valgrind testing
#define pma_get_root(x)  NULL       // with standard
#define pma_set_root(x)  ((void)0)  // malloc/free.
#endif // NO_PMA

typedef struct ln { struct ln *next; long val; char key[]; } ln_t;

static ln_t **find(ln_t **h, char *k) {  // On return **h is sought
  while (*h && strcmp((*h)->key, k))     // node, or *h == NULL if
    h = &((*h)->next);                   // not found.
  return h; }

static void set(ln_t **h, char *k, long v) {
  ln_t **p = find(h, k);  // insert new node if key not found
  if (!*p) { *p = (ln_t *)pma_malloc(sizeof **p + 1+strlen(k));
             assert(*p);  (*p)->next = NULL;  strcpy((*p)->key, k); }
  (*p)->val = v; }

static void print(ln_t **h, char *k) {
  ln_t **p = find(h, k);
  if (*p) printf("\"%s\" -> %ld\n", k, (*p)->val); }

static void del(ln_t **h, char *k) {
  ln_t *t, **p = find(h, k);
  if (*p) { t = *p;  *p = t->next;  pma_free(t); } }

int main(int argc, char *argv[]) {
  ln_t *L;  int i, r = pma_init(2, "heap.pma");
  if (r) { fprintf(stderr, "pma init->%d errno=%d\n", r, pma_errno);
           return 1; }
  L = (ln_t *)pma_get_root();  // list entry point; NULL for new heap
  for (i = 1; i < argc; i++) {
    char *a = argv[i], k[9];  long v;  // arg, key, value
    if      (2 == sscanf(a, "s %8s %ld", k, &v)) set  (&L, k, v);
    else if (1 == sscanf(a, "p %8s",     k    )) print(&L, k   );
    else if (1 == sscanf(a, "d %8s",     k    )) del  (&L, k   );
    else    fprintf(stderr, "bad argument \"%s\"\n", a);
  }
  pma_set_root(L);
  return 0;
}
