#!/bin/csh -f

# Test persistent memory allocator (driver).
# Copyright (C) 2022  Terence Kelly
# Contact:  tpkelly @ { acm.org, cs.princeton.edu, eecs.umich.edu }
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


alias clean 'rm -f test5 heap.pma test5.out.plain test5.out.pmem'

clean

if ("clean" == $1) then
    exit
endif

unlimit coredumpsize

if ( -e RCS && -d RCS ) then
    echo ; echo ; rcsdiff pma.c
    echo ; echo ; rcsdiff test5.c
    echo ; echo ; rcsdiff test5.csh
    echo ; echo
endif

if ( $?PMAALLWARNINGS ) then
    set warningflags = `gawk '{print $1}' gcc_warnings_final`
else
    echo
    echo '****************************************'
    echo '*** using bare minimum warning flags ***'
    echo '****************************************'
    set warningflags = '-Wall -Wextra'
endif

foreach ndebug ( '' -DNDEBUG )
    echo
    echo '====================================================================='
    echo "Valgrind test with conventional allocator, NDEBUG >$ndebug< ========="
    echo '====================================================================='
    echo
    echo ; echo '= = = = = = = = = = compile for warnings'  ; echo
    rm -f test5 ; gcc -DNO_PMA $ndebug -std=c99 -O3 $warningflags -o test5 test5.c
    echo ; echo '= = = = = = = = = = compile for debugging' ; echo
    rm -f test5 ; gcc -DNO_PMA $ndebug -std=c99 -g  $warningflags -o test5 test5.c
    echo '---------- all ----------'
    rm -f test5.out.plain
    valgrind ./test5 's a 12' 's b 34' 's c 56' 's d 78' 's e 90' \
                     'p a'    'p b'    'p c'    'p d'    'p e'    \
                     'd a'    'd b'    'd c'    'd d'    'd e'    \
                     'p a'    'p b'    'p c'    'p d'    'p e'    > test5.out.plain
    echo '---------- done ---------'
    echo
    echo '====================================================================='
    echo "test with p-mem allocator, NDEBUG: >$ndebug< ========================"
    echo '====================================================================='
    echo
    echo ; echo '= = = = = = = = = = compile for warnings'  ; echo
    rm -f test5 ; gcc $ndebug -std=c99 -O3 $warningflags -o test5 test5.c pma.c
    echo ; echo '= = = = = = = = = = compile for debugging' ; echo
    rm -f test5 ; gcc $ndebug -std=c99 -g  $warningflags -o test5 test5.c pma.c
    rm -f heap.pma test5.out.pmem
    truncate -s 409600 heap.pma
    echo '---------- set ----------' ; ./test5 's a 12' 's b 34' 's c 56' 's d 78' 's e 90'  > test5.out.pmem
    echo '--------- print ---------' ; ./test5 'p a'    'p b'    'p c'    'p d'    'p e'    >> test5.out.pmem
    echo '---------- del ----------' ; ./test5 'd a'    'd b'    'd c'    'd d'    'd e'    >> test5.out.pmem
    echo '--------- print ---------' ; ./test5 'p a'    'p b'    'p c'    'p d'    'p e'    >> test5.out.pmem
    echo '---------- done ---------'
    echo '--------- output --------'
    cat test5.out.pmem
    echo '--- diff (should be empty) ---'
    diff test5.out.plain test5.out.pmem
    set ckplain = `md5sum test5.out.plain | gawk '{print $1}'`
    set ckpmem  = `md5sum test5.out.pmem  | gawk '{print $1}'`
    if ($ckplain != $ckpmem) then
        echo
        echo '***  ERROR!  checksum mismatch!  ***'
        echo
        exit
    endif
    echo
    clean
end

